@@ -26,11 +26,13 @@
 /*	given in parentheses:
 /* .IP "no_unknown_recipient_checks (INPUT_TRANSP_UNKNOWN_RCPT)"
 /*	Do not try to reject unknown recipients.
-/* .IP "no_address_mappings (INPUT_TRANSP_ADDRESS_MAPPING)
+/* .IP "no_address_mappings (INPUT_TRANSP_ADDRESS_MAPPING)"
 /*	Disable canonical address mapping, virtual alias map expansion,
 /*	address masquerading, and automatic BCC recipients.
-/* .IP "no_header_body_checkss (INPUT_TRANSP_HEADER_BODY)
+/* .IP "no_header_body_checks (INPUT_TRANSP_HEADER_BODY)"
 /*	Disable header/body_checks.
+/* .IP "no_milters (INPUT_TRANSP_MILTER)"
+/*	Disable Milter applications.
 /*
 /*	input_transp_cleanup() takes a bunch of cleanup processing
 /*	flags and updates them according to the settings in the
@@ -71,6 +73,7 @@ int     input_transp_mask(const char *param_name, const char *pattern)
 	"no_unknown_recipient_checks", INPUT_TRANSP_UNKNOWN_RCPT,
 	"no_address_mappings", INPUT_TRANSP_ADDRESS_MAPPING,
 	"no_header_body_checks", INPUT_TRANSP_HEADER_BODY,
+	"no_milters", INPUT_TRANSP_MILTER,
 	0,
     };
 
@@ -90,6 +93,8 @@ int     input_transp_cleanup(int cleanup_flags, int transp_mask)
 	cleanup_flags &= ~(CLEANUP_FLAG_BCC_OK | CLEANUP_FLAG_MAP_OK);
     if (transp_mask & INPUT_TRANSP_HEADER_BODY)
 	cleanup_flags &= ~CLEANUP_FLAG_FILTER;
+    if (transp_mask & INPUT_TRANSP_MILTER)
+	cleanup_flags &= ~CLEANUP_FLAG_MILTER;
     if (msg_verbose)
 	msg_info("after %s: cleanup flags = %s",
 		 myname, cleanup_strflags(cleanup_flags));
